from smdc.format import Format
from smdc.note import Note
import io
import os
from smdc.format.cypher import escape_cypher
from py2neo import Graph, Node, Relationship
from py2neo.database.work import ClientError
import tqdm



class Neo4j(Format):

    def parse(self, file: io.TextIOWrapper, name, parsed_notes: [Note]) -> Note:
        raise NotImplementedError

    def write(self, parsed_notes: [Note], args):
        try:
            g = Graph(password=args.password)
        except ClientError as e:
            print(e)
            raise Exception("Please provide a password through smdc --password <pwd>!")
        tx = g.begin()
        if not args.retaindb:
            print("Clearing neo4j database")
            tx.run("MATCH (n) DETACH DELETE n")

        nodes = {}
        print("Converting nodes")
        node_graph = None
        # First create all nodes in the graph before doing the relationships, so they all exist.
        for name, note in tqdm.tqdm(parsed_notes.items()):
            tags = []
            if note.tags:
                tags = map(escape_cypher, note.tags)
            properties = {}
            for property, value in note.properties.items():
                properties[property] = escape_cypher(str(value))
            node = Node(*tags, **properties)
            nodes[name] = node
            if not node_graph:
                node_graph = node
            else:
                node_graph = node | node_graph
        tx.create(node_graph)
        rel_graph = None
        print("Creating relationships")
        i = 1
        for name, note in tqdm.tqdm(parsed_notes.items()):
            if not note.out_rels.keys():
                continue
            src_node = nodes[name]
            for trgt, rels in note.out_rels.items():
                if trgt not in nodes:
                    nodes[trgt] = Node(name=trgt)
                    # node_graph = node_graph | nodes[trgt]
                    if not rel_graph:
                        rel_graph = nodes[trgt]
                    else:
                        rel_graph = rel_graph | nodes[trgt]
                trgt_node = nodes[trgt]
                for rel in rels:
                    properties = {}
                    for property, value in rel.properties.items():
                        properties[property] = escape_cypher(str(value))
                    rel = Relationship(src_node, escape_cypher(rel.type), trgt_node, **properties)
                    if not rel_graph:
                        rel_graph = rel
                    else:
                        rel_graph = rel_graph | rel
            # Send batches to server. Greatly speeds up conversion.
            if i % args.batch_size == 0:
                tx.create(rel_graph)
                rel_graph = None
            i += 1
        if rel_graph:
            tx.create(rel_graph)
        print("Committing data")
        tx.commit()





from __future__ import annotations
import os

class Relationship:
    def __init__(self, type: str, properties={}):
        self.type = type
        self.properties = properties

    def __str__(self):
        return self.type + self.properties.__str__()


class Note:

    def __init__(self, name:str, tags: [str], content:str, properties={}, out_rels={}, in_rels={}):
        self.name = name
        self.tags = tags
        self.content = content
        self.out_rels = out_rels
        self.in_rels = in_rels
        self.properties = properties


    def add_out_rel(self, to:str, rel:Relationship):
        if to in self.out_rels:
            self.out_rels[to].append(rel)
        else:
            self.out_rels[to] = [rel]

    def add_in_rel(self, src: str, rel: Relationship):
        if src in self.in_rels:
            self.in_rels[src].append(rel)
        else:
            self.in_rels[src] = [rel]

    def __str__(self):
        return self.name + os.linesep + self.tags.__str__() + os.linesep + self.content + os.linesep + self.out_rels.__str__()



from pathlib import Path
from smdc.format import Format
from tqdm import tqdm
import os

def obsidian_url(name:str, vault:str) -> str:
    return "obsidian://open?vault=" + vault + "&file=" + name.replace(" ", "%20") + ".md"

def parse_folder(format: Format, notes_path="", recursive=True, note_extension='.md', vault_name="", add_obsidian_url=True):
    if recursive:
        iterate = Path(notes_path).rglob("*" + note_extension)
    else:
        iterate = Path(notes_path).glob("*" + note_extension)
    all_files = list(iterate)
    parsed_notes = {}
    print("Parsing notes")
    for path in tqdm(all_files):
        with open(path, mode='r', encoding='utf-8') as f:
            name = os.path.basename(path)[:-len(note_extension)]
            try:
                note = format.parse(f, name, parsed_notes)
                note.properties["obsidian_url"] = obsidian_url(name, vault_name)
                parsed_notes[name] = note
            except Exception as e:
                print(e)
                print("Exception raised during parsing " + path + ". Skipping this note! Please report this.")
    return parsed_notes




import smdc.parse as parse
from smdc.format import FORMAT_TYPES
import argparse


def convert(args):
    vault_name = args.vault_name
    if not vault_name:
        vault_name = args.input
    notes = parse.parse_folder(FORMAT_TYPES[args.input_format], notes_path=args.input, recursive=args.r,
                               note_extension=args.extension, vault_name=vault_name)
    FORMAT_TYPES[args.output_format].write(notes, args)


def main():
    parser = argparse.ArgumentParser(
        description='Convert different typed links representations of Markdown into other formats.')
    parser.add_argument('--input', metavar='i', type=str, default="markdown",
                        help='directory with markdown files')
    parser.add_argument('--output', metavar='o', type=str, default="out",
                        help='directory for output files')
    parser.add_argument('--input_format', metavar='f', type=str, default="typed_list",
                        help='format of inputs')
    parser.add_argument('--output_format', metavar='r', type=str, default="neo4j",
                        help='format of inputs')
    parser.add_argument('--extension', metavar='e', type=str, default=".md",
                        help='extension of markdown files')
    parser.add_argument('--retaindb', action="store_true", default=False)
    parser.add_argument('--password', metavar='p', type=str, default="")
    parser.add_argument("--vault_name", metavar='n', type=str, default=None, help="Defaults to input directory name")
    parser.add_argument("--batch_size", metavar='b', type=int, default=75, help="Batch size for sending to neo4j")
    parser.add_argument('-r', action="store_false", default=True)
    args = parser.parse_args()
    convert(args)


if __name__ == "__main__":
    main()



from smdc.format import Format
from smdc.note import Note, Relationship
import io
import os
from smdc.format.util import escape_quotes


class CSV(Format):

    def parse(self, file: io.TextIOWrapper, name, parsed_notes: [Note]) -> Note:
        raise NotImplementedError

    def write(self, file, parsed_notes: [Note]):
        # First create all nodes in the graph before doing the relationships, so they all exist.
        with open(file + '.csv', 'w') as f:

            for name, note in parsed_notes.items():

                line = "CREATE ("
                if note.tags:
                    line += ":" + ":".join(note.tags)
                line += " { name: '" + escape_quotes(name) + "', content: '" + escape_quotes(note.content) + "'"
                for property, value in note.properties.items():
                    line += ", " + property + ": '" + escape_quotes(str(value)) + "'"
                line += "});" + os.linesep
                f.write(line)

            for name, note in parsed_notes.items():
                f.write("MATCH (a)" + os.linesep + "WHERE a.name == '" + escape_quotes(name) + "'" + os.linesep)
                i = 0
                for trgt, rels in note.out_rels.items():
                    f.write(
                        "MATCH (b" + str(i) + ")" + os.linesep + "WHERE b" + str(i) + ".name == '" +
                        escape_quotes(trgt) + "'" + os.linesep)
                    for rel in rels:
                        f.write("CREATE (a)-[r:" + rel.type + "]->(b" + str(i) + ")" + os.linesep)
                    i += 1
                f.write(";" + os.linesep)




import abc
from smdc.note import Note
import io

class Format(abc.ABC):

    @abc.abstractmethod
    def parse(self, file: io.TextIOWrapper, name: str, parsed_notes: [Note]) -> Note:
        ...

    @abc.abstractmethod
    def write(self, parsed_notes: [Note], args):
        ...


from setuptools import setup, find_packages

with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


setup(name='semantic-markdown-converter',
      version='0.2.1',
      description='Converts different typed link formats in Markdown into each other and to external formats.',
      long_description=long_description,
      long_description_content_type="text/markdown",
      url='https://github.com/HEmile/semantic-markdown-converter',
      packages=find_packages(),
      install_requires=['pyyaml', 'tqdm', 'py2neo'],
      entry_points={
          'console_scripts': ['smdc=smdc.convert:main']
      },
      classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
      ],
      author='Emile van Krieken',
      author_email='emilevankrieken@live.nl',
      license='MIT',
      zip_safe=False,
      python_requires='>=3.5',)